package org.ids_mannheim;

import java.util.Arrays;
import java.util.OptionalInt;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.stream.IntStream;

public class WorkerNodePool {
    CopyOnWriteArrayList<String> execPool = new CopyOnWriteArrayList<>();
    CopyOnWriteArrayList<String> hostPool = new CopyOnWriteArrayList<>();
    CopyOnWriteArrayList<Boolean> busy = new CopyOnWriteArrayList<>();

    public final int size;

    public String getExec(int i) {
        return execPool.get(i);
    }

    public String getHost(int i) {
        return hostPool.get(i);
    }

    private String hostToExec(String host) {
        if (host.equals("") || host.matches("local")) {
            return "nice -5 ";
        } else {
            return " ssh " + host + " nice -5 ";
        }
    }

    public void markFree(int i) {
        busy.set(i, false);
    }

    public synchronized int getNextFree() throws InterruptedException {
        while (true) {
            OptionalInt result = IntStream.range(0, size).filter(i -> !busy.get(i)).findFirst();
            if (result.isPresent()) {
                int i = result.getAsInt();
                busy.set(i, true);
                return i;
            } else {
                TotalNGram.logger.warning("No free worker node. This should not happen.");
                //noinspection BusyWait
                Thread.sleep(1000);
            }
        }
    }

    public WorkerNodePool(String description) {
        Arrays.stream(description.split(", *")).forEachOrdered(s -> {
            String[] single = s.split("\\s*[*]\\s*");
            final String host;
            int procs = 0;
            if (single.length == 1) {
                procs = 1;
                host = single[0];
            } else if (single.length == 2) {
                procs = Integer.parseInt(single[0]);
                host = single[1];
            } else {
                host = "";
                TotalNGram.logger.severe("Wrong worker node format: " + s);
                System.exit(-1);
            }
            IntStream.range(0, procs).forEach(u -> {
                hostPool.add(host);
                execPool.add(hostToExec(host));
                busy.add(false);
            });

        });
        size = hostPool.size();
    }
}
